import { GeoAPIResponse, GeolocationCoordinates } from './GeoApiClient.types.ts'

export interface ResolvesCoordinates {
  resolveCoordinates(geolocationId: string): Promise<GeolocationCoordinates>
}

export class GeoApiClient implements ResolvesCoordinates {
  constructor(private readonly endpointUrl: string) {}

  async resolveCoordinates(
    geolocationId: string,
  ): Promise<GeolocationCoordinates> {
    const response = await this.fetchPlaceDetails(geolocationId)
    const { lat, lon } = response.data?.placeDetails?.geolocation?.properties || {}
    if (lat && lon) {
      return { lat, lon }
    } else {
      throw new Deno.errors.NotCapable(
        `Resolution of lat/lon failed (no data included in response for geolocationId=${geolocationId})`,
      )
    }
  }

  private async fetchPlaceDetails(
    geolocationId: string,
  ): Promise<GeoAPIResponse> {
    const graphQLQuery = `{ placeDetails(id: "${geolocationId}") { geolocation } }`
    const response = await fetch(this.endpointUrl, {
      'body': JSON.stringify({ query: graphQLQuery }),
      'headers': {
        'Accept': 'application/graphql-response+json, application/json',
        'Content-Type': 'application/json',
      },
      'method': 'POST',
    })
    return await response.json() as GeoAPIResponse
  }
}
